#pragma once

#include "RawBuffer.hpp"
#include "depthai-shared/common/Point3f.hpp"
#include "depthai-shared/common/Timestamp.hpp"

namespace dai {

struct IMUReport {
    enum class IMUReportAccuracy : std::uint8_t {
        UNRELIABLE = 0,
        LOW = 1,
        MEDIUM = 2,
        HIGH = 3,
    };
    /** @brief 8-bit unsigned integer used to track reports.
     *
     * The sequence number increments once for each report sent.  Gaps
     * in the sequence numbers indicate missing or dropped reports.
     */
    int32_t sequence = 0;

    /** Accuracy of sensor */
    IMUReportAccuracy accuracy = IMUReportAccuracy::UNRELIABLE;

    Timestamp timestamp = {};
};
NLOHMANN_DEFINE_TYPE_NON_INTRUSIVE(IMUReport, sequence, accuracy, timestamp);

/**
 * @brief Accelerometer
 *
 * See the SH-2 Reference Manual for more detail.
 */
struct IMUReportAccelerometer : IMUReport {
    float x = 0;
    float y = 0;
    float z = 0;
};
NLOHMANN_DEFINE_TYPE_NON_INTRUSIVE(IMUReportAccelerometer, x, y, z, sequence, accuracy, timestamp);

/**
 * @brief Gyroscope
 *
 * See the SH-2 Reference Manual for more detail.
 */
struct IMUReportGyroscope : IMUReport {
    /* Units are rad/s */
    float x = 0;
    float y = 0;
    float z = 0;
};
NLOHMANN_DEFINE_TYPE_NON_INTRUSIVE(IMUReportGyroscope, x, y, z, sequence, accuracy, timestamp);

/**
 * @brief Uncalibrated gyroscope
 *
 * See the SH-2 Reference Manual for more detail.
 */
struct IMUReportGyroscopeUncalibrated : IMUReport {
    /* Units are rad/s */
    float x = 0;     /**< @brief [rad/s] */
    float y = 0;     /**< @brief [rad/s] */
    float z = 0;     /**< @brief [rad/s] */
    float biasX = 0; /**< @brief [rad/s] */
    float biasY = 0; /**< @brief [rad/s] */
    float biasZ = 0; /**< @brief [rad/s] */
};
NLOHMANN_DEFINE_TYPE_NON_INTRUSIVE(IMUReportGyroscopeUncalibrated, x, y, z, biasX, biasY, biasZ, sequence, accuracy, timestamp);

/**
 * @brief Magnetic field
 *
 * See the SH-2 Reference Manual for more detail.
 */
struct IMUReportMagneticField : IMUReport {
    /* Units are uTesla */
    float x = 0; /**< @brief [uTesla] */
    float y = 0; /**< @brief [uTesla] */
    float z = 0; /**< @brief [uTesla] */
};
NLOHMANN_DEFINE_TYPE_NON_INTRUSIVE(IMUReportMagneticField, x, y, z, sequence, accuracy, timestamp);

/**
 * @brief Uncalibrated magnetic field
 *
 * See the SH-2 Reference Manual for more detail.
 */
struct IMUReportMagneticFieldUncalibrated : IMUReport {
    /* Units are uTesla */
    float x = 0;     /**< @brief [uTesla] */
    float y = 0;     /**< @brief [uTesla] */
    float z = 0;     /**< @brief [uTesla] */
    float biasX = 0; /**< @brief [uTesla] */
    float biasY = 0; /**< @brief [uTesla] */
    float biasZ = 0; /**< @brief [uTesla] */
};
NLOHMANN_DEFINE_TYPE_NON_INTRUSIVE(IMUReportMagneticFieldUncalibrated, x, y, z, biasX, biasY, biasZ, sequence, accuracy, timestamp);

/**
 * @brief Rotation Vector with Accuracy
 *
 * See the SH-2 Reference Manual for more detail.
 */
struct IMUReportRotationVectorWAcc : IMUReport {
    float i = 0;        /**< @brief Quaternion component i */
    float j = 0;        /**< @brief Quaternion component j */
    float k = 0;        /**< @brief Quaternion component k */
    float real = 0;     /**< @brief Quaternion component, real */
    float accuracy = 0; /**< @brief Accuracy estimate [radians] */
};
NLOHMANN_DEFINE_TYPE_NON_INTRUSIVE(IMUReportRotationVectorWAcc, i, j, k, real, accuracy, sequence, accuracy, timestamp);

/**
 * @brief Rotation Vector
 *
 * See the SH-2 Reference Manual for more detail.
 */
struct IMUReportRotationVector : IMUReport {
    float i = 0;    /**< @brief Quaternion component i */
    float j = 0;    /**< @brief Quaternion component j */
    float k = 0;    /**< @brief Quaternion component k */
    float real = 0; /**< @brief Quaternion component real */
};
NLOHMANN_DEFINE_TYPE_NON_INTRUSIVE(IMUReportRotationVector, i, j, k, real, sequence, accuracy, timestamp);

/**
 * @brief heartRateMonitor
 *
 * See SH-2 Reference Manual for details.
 */
struct IMUReportGyroIntegratedRV : IMUReport {
    float i = 0;       /**< @brief Quaternion component i */
    float j = 0;       /**< @brief Quaternion component j */
    float k = 0;       /**< @brief Quaternion component k */
    float real = 0;    /**< @brief Quaternion component real */
    float angVelX = 0; /**< @brief Angular velocity about x [rad/s] */
    float angVelY = 0; /**< @brief Angular velocity about y [rad/s] */
    float angVelZ = 0; /**< @brief Angular velocity about z [rad/s] */
};
NLOHMANN_DEFINE_TYPE_NON_INTRUSIVE(IMUReportGyroIntegratedRV, i, j, k, real, angVelX, angVelY, angVelZ, sequence, accuracy, timestamp);

/**
 * IMU output
 *
 * Contains configuration data, average depth for the calculated ROI on depth map.
 * Together with spatial coordinates: x,y,z relative to the center of depth map.
 * Units are in millimeters.
 */
struct IMUDatas {
    IMUReportAccelerometer rawAcceleroMeter;

    IMUReportAccelerometer acceleroMeter;
    IMUReportAccelerometer linearAcceleroMeter;
    IMUReportAccelerometer gravity;

    IMUReportGyroscope rawGyroscope;
    IMUReportGyroscope gyroscope;
    IMUReportGyroscopeUncalibrated gyroscopeUncalibrated;

    IMUReportMagneticField rawMagneticField;
    IMUReportMagneticField magneticField;
    IMUReportMagneticFieldUncalibrated magneticFieldUncalibrated;

    IMUReportRotationVectorWAcc rotationVector;
    IMUReportRotationVector gameRotationVector;
    IMUReportRotationVectorWAcc geoMagRotationVector;

    IMUReportRotationVectorWAcc arvrStabilizedRotationVector;
    IMUReportRotationVector arvrStabilizedGameRotationVector;
    IMUReportGyroIntegratedRV gyroIntegratedRotationVector;
};

NLOHMANN_DEFINE_TYPE_NON_INTRUSIVE(IMUDatas,
                                   rawAcceleroMeter,
                                   acceleroMeter,
                                   linearAcceleroMeter,
                                   gravity,
                                   rawGyroscope,
                                   gyroscope,
                                   gyroscopeUncalibrated,
                                   rawMagneticField,
                                   magneticField,
                                   magneticFieldUncalibrated,
                                   rotationVector,
                                   gameRotationVector,
                                   geoMagRotationVector,
                                   arvrStabilizedRotationVector,
                                   arvrStabilizedGameRotationVector,
                                   gyroIntegratedRotationVector);

struct RawIMUData : public RawBuffer {
    std::vector<IMUDatas> imuDatas;

    void serialize(std::vector<std::uint8_t>& metadata, DatatypeEnum& datatype) override {
        nlohmann::json j = *this;
        metadata = nlohmann::json::to_msgpack(j);
        datatype = DatatypeEnum::IMUDatas;
    };

    NLOHMANN_DEFINE_TYPE_INTRUSIVE(RawIMUData, imuDatas);
};

}  // namespace dai
